SPELLAUTOCOMPLETE_MAX_BUTTONS = 12;

	
AUTOCOMPLETE_SIMPLE_REGEX = "(.+)";
AUTOCOMPLETE_SIMPLE_FORMAT_REGEX = "%1$s";

AUTOCOMPLETE_DEFAULT_Y_OFFSET = 3;


local SPELLAUTOCOMPLETE_LIST = {}
local SPELLAUTOCOMPLETE_MAP = {}
local count = 1;
local name
for i = 1, 1000000 do
	name = GetSpellInfo(i);
	if name and not SPELLAUTOCOMPLETE_MAP[name] and GetSpellLink(i) then
	
		print(GetSpellLink(i),GetSpellInfo(i))
		SPELLAUTOCOMPLETE_MAP[name] = i
		SPELLAUTOCOMPLETE_LIST[count] = name
		count = count + 1
	end
end

table.sort(SPELLAUTOCOMPLETE_LIST)
	
local function GetSpellAutoCompleteResults(text)
	local results = {}
	local resultsCount = 1
	local SPELL_COUNT = #SPELLAUTOCOMPLETE_LIST
	for i,spellName in ipairs(SPELLAUTOCOMPLETE_LIST) do
		pcall(function() 
			if string.find(spellName:lower(),text:lower()) == 1 then
				results[resultsCount] = spellName;
				resultsCount = resultsCount + 1
			end
		end)		
	end
	
	return results;
end

function SpellAutoComplete_OnLoad(self)
	self:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
	self:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
	
	self.maxHeight = SPELLAUTOCOMPLETE_MAX_BUTTONS * AutoCompleteButton1:GetHeight();
	
	AutoCompleteInstructions:SetText("|cffbbbbbb"..PRESS_TAB.."|r");
end

function SpellAutoComplete_Update(parent, text, cursorPosition)
	local self = AutoCompleteBox;
	local attachPoint;
	if ( not text or text == "" ) then
		SpellAutoComplete_HideIfAttachedTo(parent);
		return;
	end
	if ( cursorPosition <= strlen(text) ) then
		self:SetParent(parent);
		if(self.parent ~= parent) then
			SpellAutoComplete_SetSelectedIndex(self, 0);
			self.parentArrows = parent:GetAltArrowKeyMode();
		end
		parent:SetAltArrowKeyMode(false);
		
		if ( parent:GetBottom() - self.maxHeight <= (AUTOCOMPLETE_DEFAULT_Y_OFFSET + 10) ) then	--10 is a magic number from the offset of AutoCompleteButton1.
			attachPoint = "ABOVE";
		else
			attachPoint = "BELOW";
		end
		if ( (self.parent ~= parent) or (self.attachPoint ~= attachPoint) ) then
			if ( attachPoint == "ABOVE" ) then
				self:ClearAllPoints();
				self:SetPoint("BOTTOMLEFT", parent, "TOPLEFT", parent.autoCompleteXOffset or 0, parent.autoCompleteYOffset or -AUTOCOMPLETE_DEFAULT_Y_OFFSET);
			elseif ( attachPoint == "BELOW" ) then
				self:ClearAllPoints();
				self:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", parent.autoCompleteXOffset or 0, parent.autoCompleteYOffset or AUTOCOMPLETE_DEFAULT_Y_OFFSET);
			end
			self.attachPoint = attachPoint;
		end
		
		self.parent = parent;
		--We ask for one more result than we need so that we know whether or not results are continued
		local possibilities = GetSpellAutoCompleteResults(text);

		if (not possibilities) then
			possibilities = {};
		end
		SpellAutoComplete_UpdateResults(self, possibilities, parent.autoCompleteContext);
	else
		SpellAutoComplete_HideIfAttachedTo(parent);
	end
end

function SpellAutoComplete_HideIfAttachedTo(parent)
	local self = AutoCompleteBox;
	if ( self.parent == parent ) then
		if( self.parentArrows ) then
			parent:SetAltArrowKeyMode(self.parentArrows);
			self.parentArrows = nil;
		end
		self.parent = nil;
		self:Hide();
	end
end

function SpellAutoComplete_SetSelectedIndex(self, index)
	self.selectedIndex = index;
	for i=1, SPELLAUTOCOMPLETE_MAX_BUTTONS do
		_G["AutoCompleteButton"..i]:UnlockHighlight();
	end
	if ( index ~= 0 ) then
		_G["AutoCompleteButton"..index]:LockHighlight();
	end
end

function SpellAutoComplete_GetSelectedIndex(self)
	return self.selectedIndex;
end

function SpellAutoComplete_GetNumResults(self)
	return self.numResults;
end

function SpellAutoComplete_UpdateResults(self, results, context)
	local totalReturns = #results;
	local numReturns = min(totalReturns, SPELLAUTOCOMPLETE_MAX_BUTTONS);
	local maxWidth = 150;
	for i=1, numReturns do
		local button = _G["AutoCompleteButton"..i]
		button.name = Ambiguate(results[i], "none");
		button:SetText(results[i]);
		maxWidth = max(maxWidth, button:GetFontString():GetWidth()+30);
		button:Enable();
		button:Show();
	end
	
	for i = numReturns+1, SPELLAUTOCOMPLETE_MAX_BUTTONS do
		_G["AutoCompleteButton"..i]:Hide();
	end
	
	if ( numReturns > 0 ) then
		maxWidth = max(maxWidth, AutoCompleteInstructions:GetStringWidth()+30);
		self:SetHeight(numReturns*AutoCompleteButton1:GetHeight()+35);
		self:SetWidth(maxWidth);
		self:Show();
		SpellAutoComplete_SetSelectedIndex(self, 1);
	else
		self:Hide();
	end
		
	if ( totalReturns > SPELLAUTOCOMPLETE_MAX_BUTTONS )  then
		local button = _G["AutoCompleteButton"..SPELLAUTOCOMPLETE_MAX_BUTTONS];
		button:SetText(CONTINUED);
		button:Disable();
		self.numResults = numReturns - 1;
	else 
		self.numResults = numReturns;
	end
end

function SpellAutoComplete_IncrementSelection(editBox, up)
	local autoComplete = AutoCompleteBox;
	if ( autoComplete:IsShown() and autoComplete.parent == editBox ) then
		local selectedIndex = SpellAutoComplete_GetSelectedIndex(autoComplete);
		local numReturns = SpellAutoComplete_GetNumResults(autoComplete);
		if ( up ) then
			local nextNum = mod(selectedIndex - 1, numReturns);
			if ( nextNum <= 0 ) then
				nextNum = numReturns;
			end
			SpellAutoComplete_SetSelectedIndex(autoComplete, nextNum);
		else
			local nextNum = mod(selectedIndex + 1, numReturns);
			if ( nextNum == 0 ) then
				nextNum = numReturns;
			end
			SpellAutoComplete_SetSelectedIndex(autoComplete, nextNum)
		end
		return true;
	end
	return false;
end

function SpellAutoCompleteEditBox_OnTabPressed(editBox)
	return SpellAutoComplete_IncrementSelection(editBox, IsShiftKeyDown())
end

function SpellAutoCompleteEditBox_OnArrowPressed(self, key)
	if ( key == "UP" ) then
		return SpellAutoComplete_IncrementSelection(self, true);
	elseif ( key == "DOWN" ) then
		return SpellAutoComplete_IncrementSelection(self, false);
	end
end

function SpellAutoCompleteEditBox_OnEnterPressed(self)
	local autoComplete = AutoCompleteBox;
	if ( autoComplete:IsShown() and (autoComplete.parent == self) and (SpellAutoComplete_GetSelectedIndex(autoComplete) ~= 0) ) then
		SpellAutoCompleteButton_OnClick(_G["AutoCompleteButton"..SpellAutoComplete_GetSelectedIndex(autoComplete)]);
		return true;
	end
	return false;
end

function SpellAutoCompleteEditBox_OnTextChanged(self, userInput)
    if ( userInput ) then
        SpellAutoComplete_Update(self, self:GetText(), self:GetUTF8CursorPosition());
    end
    if(self:GetText() == "") then
        SpellAutoComplete_HideIfAttachedTo(self);
    end
end

function SpellAutoCompleteEditBox_AddHighlightedText(editBox, text)
	local editBoxText = editBox:GetText();
	local utf8Position = editBox:GetUTF8CursorPosition();
	local possibilities = GetSpellAutoCompleteResults(text);
	
	if ( possibilities and possibilities[1] ) then
		--We're going to be setting the text programatically which will clear the userInput flag on the editBox. So we want to manually update the dropdown before we change the text.
		SpellAutoComplete_Update(editBox, editBoxText, utf8Position);
		local newText = string.gsub(editBoxText, AUTOCOMPLETE_SIMPLE_REGEX,
							string.format(AUTOCOMPLETE_SIMPLE_FORMAT_REGEX, possibilities[1],
								string.match(editBoxText, AUTOCOMPLETE_SIMPLE_REGEX)),
								1)
		editBox:SetText(newText);
		editBox:HighlightText(strlen(editBoxText), strlen(newText));	--This won't work if there is more after the name, but we aren't enabling this for normal chat (yet). Please fix me when we do.
		editBox:SetCursorPosition(strlen(editBoxText));
	end
end

function SpellAutoCompleteEditBox_OnChar(self)
	if (self.addHighlightedText and self:GetUTF8CursorPosition() == strlenutf8(self:GetText())) then
		SpellAutoCompleteEditBox_AddHighlightedText(self, self:GetText());
	end
end

function SpellAutoCompleteEditBox_OnEditFocusLost(self)
	SpellAutoComplete_HideIfAttachedTo(self);
end

function SpellAutoCompleteEditBox_OnEscapePressed(self)
	local autoComplete = AutoCompleteBox;
	if ( autoComplete:IsShown() and autoComplete.parent == self ) then
		SpellAutoComplete_HideIfAttachedTo(self);
		return true;
	end
	return false;
end	

function SpellAutoCompleteButton_OnClick(self)
	local autoComplete = self:GetParent();
	local editBox = autoComplete.parent;
	local editBoxText = editBox:GetText();
	local newText;
	
	if (editBox.command) then
		newText = editBox.command.." "..self.name;
	else
		newText = string.gsub(editBoxText, AUTOCOMPLETE_SIMPLE_REGEX,
			string.format(AUTOCOMPLETE_SIMPLE_FORMAT_REGEX, self.name,
				string.match(editBoxText, AUTOCOMPLETE_SIMPLE_REGEX)),
				1);
	end
	
	if ( editBox.addSpaceToAutoComplete ) then
		newText = newText.." ";
	end
	
	editBox:SetText(newText);
	--When we change the text, we move to the end, so we'll be consistent and move to the end if we don't change it as well.
	editBox:SetCursorPosition(strlen(newText));
	autoComplete:Hide();
end